/*
 ATtiny84 Si4432 RX Test
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/02/2017
 Last Modified: 3/02/2017
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef bool
	typedef uint8_t bool;
#endif 

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Request Types
#define CHECK_IN 1
#define CHECK_IN_REPLY 2
#define REQUEST_RANDOM_NO 3
#define SENSOR_UPDATE 4

uint8_t dataIn[64];

extern uint8_t RH_RF22_init(void);

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// Initialise the SPI / SS
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	DDRA &= ~(1<<PA6); // SPI DI
	PORTA |= (1<<PA6); // SPI DI
	
	DDRA |= (1<<SS);
	si4432_SS_hi;
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

void clear_data_in(void) {
	for (uint8_t x = 0; x < 64; x++) {
		dataIn[x] = 0;
	}
}

void wait_for_valid_packet(void) {
	RH_RF22_resetFifos();
	RH_RF22_setModeRx();
	
	// Wait for a packet that matches our address
	uint8_t readReg = 0;
	uint8_t timeoutCounter = 0;
	while (!(readReg & ipkvalid)) {
		readReg = spi_read_reg(RH_RF22_REG_03_INTERRUPT_STATUS1);
		_delay_ms(5);
		
		timeoutCounter++;
		if (timeoutCounter >= 50) {
			RH_RF22_setModeRx();
			RH_RF22_resetFifos();
		}
	}
}

// Setup
void setup(void) {
	_delay_ms(250); // Wait a little bit after power on
	spi_init(); // Initialise SPI
	
	DDRA |= (1<<PA0);
	
	if (RH_RF22_init()) {
		PORTA |= (1<<PA0);
		_delay_ms(50);
		PORTA &= ~(1<<PA0);
	}
	
	sei(); // Turn on interrupts
}
