/*
 ATtiny84 Si4432 TX Test
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/02/2017
 Last Modified: 3/02/2017
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define SS  PA2
#define si4432_SS_hi	PORTA |=  (1<<SS);
#define si4432_SS_lo	PORTA &= ~(1<<SS);

extern uint8_t RH_RF22_init(void);

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// Initialise the SPI / SS
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	DDRA &= ~(1<<PA6); // SPI DI
	PORTA |= (1<<PA6); // SPI DI
	
	DDRA |= (1<<SS);
	si4432_SS_hi;
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup
void setup(void) {
	_delay_ms(250); // Wait a little bit after power on
	spi_init(); // Initialise SPI
	
	DDRA |= (1<<PA0);
	
	if (RH_RF22_init()) {
		PORTA |= (1<<PA0);
		_delay_ms(50);
		PORTA &= ~(1<<PA0);
	}
	
	sei(); // Turn on interrupts
}
